/*
 * File: ChaosGame.java
 * ================================================================
 * An implementation of the Chaos Game.  The program picks three
 * points to act as vertices of a triangle (these will be the
 * upper-left, lower-right, and lower-left corners of the window).
 * Starting with the top point, we repeatedly do the following:
 * 
 * 1. Choose a random one of the three corners.
 * 2. Move halfway between our current location and that corner.
 * 3. Put a dot at our location.
 * 
 * The result is pretty surprising!
 */
import acm.graphics.*;
import acm.program.*;
import acm.util.*;
import java.awt.*;

public class ChaosGame extends GraphicsProgram {	
	/**
	 * Runs the chaos game.
	 */
	public void run() {
		/* For elegance, let's make the background black. */
		setBackground(Color.BLACK);
	
		GPoint pt = new GPoint(0, 0);
		while (true) {
			moveRandomly(pt);
			plotPixel(pt);
		}
	}
	
	/**
	 * Chooses a random corner position so that we can move halfway between the
	 * current position and the corner position.
	 * 
	 * @return A random corner of the screen.
	 */
	private GPoint chooseRandomPosition() {
		/* Choose an index between 1 and 3, inclusive. */
		RandomGenerator rgen = RandomGenerator.getInstance();
		int choice = rgen.nextInt(1, 3);
		
		/* Based on the index, return one of the corners. */
		if (choice == 1) {
			return new GPoint(0, 0);
		} else if (choice == 2) {
			return new GPoint(0, getHeight());
		} else {
			return new GPoint(getWidth(), getHeight());
		}
	}
	
	/**
	 * Moves the specified point halfway to a randomly-chosen corner of the screen.
	 * 
	 * @param pt The point to move.
	 */
	private void moveRandomly(GPoint pt) {
		GPoint newPt = chooseRandomPosition();
		
		pt.setLocation((pt.getX() + newPt.getX()) / 2.0,
				       (pt.getY() + newPt.getY()) / 2.0);
		
	}
	
	/**
	 * Plots a randomly-colored pixel at the specified location.
	 * 
	 * @param pt The point at which to plot the pixel.
	 */
	private void plotPixel(GPoint pt) {
		/* Create a 1 x 1 rectangle to use as our pixel. */
		GRect pixel = new GRect(pt.getX(), pt.getY(), 1, 1);
		pixel.setFilled(true);
		
		/* Randomly color it. */
		RandomGenerator rgen = RandomGenerator.getInstance();
		pixel.setColor(rgen.nextColor());
		
		add(pixel);
	}
}
